//#define DISABLE_VIDEO
//#define DISABLE_AUDIO

#include <QTextStream>
#include <QFileDialog>

#include <dshow/AudioCapture.h>
#include <dshow/VideoCapture.h>

#include "ffmpeg_dbg.h"
#include "captureui.h"

captureUI::captureUI(QWidget *parent, Qt::WFlags flags)
    : QMainWindow(parent, flags)
	,capturing(false)
	,hCliThread(NULL)
	,hStopCapture(NULL)
	,hCaptureStopped(NULL)
{
	ui.setupUi(this);

	output_path = "T:/test/divx";
	ui.editOutputDir->insert( output_path );

	on_buttonRefreshVideo_clicked();
	on_buttonRefreshAudio_clicked();
}

captureUI::~captureUI()
{
	StopCapture();
}

void captureUI::StopCapture()
{
	if ( capturing )
	{
		ui.buttonCapture->setDisabled(true);

		// clean ending of the capture
		SetEvent(hStopCapture);

		WaitForSingleObject( hCaptureStopped, 6000 );

		if (hCliThread != NULL)
			CloseHandle( hCliThread );
		if (hStopCapture != NULL)
			CloseHandle( hStopCapture );
		if (hCaptureStopped != NULL)
			CloseHandle( hCaptureStopped );

		ui.buttonCapture->setDisabled(false);
	}
}


void captureUI::on_buttonRefreshVideo_clicked()
{
	ui.comboCaptureVideo->clear();

	StringList device_list;

	size_t list_size = DShowVideoCaptureGraph::GetDeviceList( device_list );

	if (list_size)
	{
		bool b_last_selection_still = false;
		for (size_t i=0; i<list_size; i++)
		{
			QString deviceStr = QString::fromStdWString(device_list[i]);
			ui.comboCaptureVideo->insertItem( i, deviceStr );
			if (video_device == deviceStr)
				b_last_selection_still = true;
		}
		if (!b_last_selection_still)
			on_comboCaptureVideo_activated( ui.comboCaptureVideo->currentText() );
	}
}

void captureUI::on_buttonRefreshAudio_clicked()
{
	ui.comboCaptureAudio->clear();

	StringList device_list;

	size_t list_size = DShowAudioCaptureGraph::GetDeviceList( device_list );

	if (list_size)
	{
		bool b_last_selection_still = false;
		for (size_t i=0; i<list_size; i++)
		{
			QString deviceStr = QString::fromStdWString(device_list[i]);
			ui.comboCaptureAudio->insertItem( i, deviceStr );
			if (audio_device == deviceStr)
				b_last_selection_still = true;
		}
		if (!b_last_selection_still)
			on_comboCaptureAudio_activated( ui.comboCaptureAudio->currentText() );
	}
}

extern "C" {
	extern DWORD WINAPI cli_main( LPVOID lpParameter );
};

void captureUI::on_buttonCapture_clicked()
{
	if ( capturing )
	{
		ui.buttonCapture->setText( "Start Capture" );
		StopCapture();
	}
	else
	{
		// prepare the command-line string to grab with ffmpeg
		QString tmpStr;
		int av_argc = 0;
		const char **av_argv = (const char **)av_malloc(32 * sizeof(char *));

		av_argv[av_argc++] = "0";

#ifndef DISABLE_VIDEO // enable/disable video capture (for testing the graphs)
		// Video Settings
		av_argv[av_argc++] = "-vcodec";
		av_argv[av_argc++] = "divx";
		av_argv[av_argc++] = "-b";
		av_argv[av_argc++] = "800";
		av_argv[av_argc++] = "-pix_fmt";
		av_argv[av_argc++] = "yuv420p";

		tmpStr = ui.comboCaptureVideo->currentText();
		if (tmpStr != "") {
			av_argv[av_argc++] = "-vd";
			tmpStr += "%";
			tmpStr += ui.comboVideoInput->currentText();
			av_argv[av_argc++] = av_strdup(tmpStr.toUtf8());
		}

		tmpStr = ui.comboVideoNative->currentText();
		if (tmpStr != "") {
			av_argv[av_argc++] = "-s";
			av_argv[av_argc++] = av_strdup(tmpStr.toUtf8());
		}
		
		tmpStr = ui.comboVideoFramerate->currentText();
		if (tmpStr != "") {
			av_argv[av_argc++] = "-r";
			av_argv[av_argc++] = av_strdup(tmpStr.toUtf8());
		}
		tmpStr = ui.comboVideoFormat->currentText();
		tmpStr = tmpStr.toLower();
		if (tmpStr == "pal" || tmpStr == "ntsc") {
			// only formats supported by ffmpeg
			av_argv[av_argc++] = "-tvstd";
			av_argv[av_argc++] = av_strdup(tmpStr.toUtf8());
		}
#endif
#ifndef DISABLE_AUDIO // enable/disable video capture (for testing the graphs)
		// Audio Settings
		av_argv[av_argc++] = "-acodec";
		av_argv[av_argc++] = "mp3";
		av_argv[av_argc++] = "-ab";
		av_argv[av_argc++] = "128";

		tmpStr = ui.comboCaptureAudio->currentText();
		if (tmpStr != "") {
			av_argv[av_argc++] = "-ad";
			tmpStr += "%";
			tmpStr += ui.comboAudioInput->currentText();
			av_argv[av_argc++] = av_strdup(tmpStr.toUtf8());
		}
		tmpStr = ui.comboAudioChannels->currentText();
		if (tmpStr != "") {
			av_argv[av_argc++] = "-ac";
			av_argv[av_argc++] = av_strdup(tmpStr.toUtf8());
		}

		tmpStr = ui.comboAudioSample->currentText();
		if (tmpStr != "") {
			av_argv[av_argc++] = "-ar";
			av_argv[av_argc++] = av_strdup(tmpStr.toUtf8());
		}
#endif

		// General Settings
		av_argv[av_argc++] = "-v";
		av_argv[av_argc++] = "3";

		tmpStr = output_path;
#ifdef DISABLE_VIDEO
		tmpStr += "/cap.mp3";
#else
		tmpStr += "/cap.avi";
#endif
		av_argv[av_argc++] = av_strdup(tmpStr.toUtf8());
		
		tmpStr.setNum(av_argc);
		av_argv[0] = av_strdup(tmpStr.toUtf8());

		hStopCapture = CreateEvent( NULL, FALSE, FALSE, "divxCaptureStop" );
		hCaptureStopped = CreateEvent( NULL, FALSE, FALSE, "divxCaptureStopped" );

		hCliThread = CreateThread( NULL, 0, cli_main, av_argv, 0, NULL);

		ui.buttonCapture->setText( "Stop Capture" );
	}

	capturing = !capturing;
}


void captureUI::on_comboCaptureVideo_activated(QString)
{
	QString devStr = ui.comboCaptureVideo->currentText();

	if (devStr != video_device)
	{
		std::wstring dev = devStr.toStdWString();
		video_device = devStr;

		// update the Video properties
		ui.comboVideoNative->clear();
		ui.comboVideoFormat->clear();
		ui.comboVideoFramerate->clear();
		ui.comboVideoInput->clear();

		QString tmpStr, tmpStr2;
		std::vector<VIDEOINFOHEADER> devicecaps;
		DShowVideoCaptureGraph::GetDeviceCaps( dev, devicecaps );

		QStringList resolutions, rates, formats, inputs;
		for (size_t i=0; i<devicecaps.size(); i++)
		{
			tmpStr = QString( "%1x%2" ).arg( devicecaps[i].bmiHeader.biWidth ).arg( devicecaps[i].bmiHeader.biHeight );
			if ( !resolutions.contains( tmpStr ) )
				resolutions.push_back( tmpStr );
/*
			for (LONGLONG rate = devicecaps[i].MinFrameInterval;
				   rate <= devicecaps[i].MaxFrameInterval;
				   rate += devicecaps[i].MinFrameInterval)
*/
			{
				LONGLONG rate = devicecaps[i].AvgTimePerFrame;
				double fps = 10000000.0 / rate;
				if (fps < 10.0)
					break;
				tmpStr.setNum(fps, 'f', 2);
				if ( !rates.contains( tmpStr ) )
					rates.push_back( tmpStr );
			}
		}

		resolutions.sort();
		ui.comboVideoNative->addItems( resolutions );
		rates.sort();
		ui.comboVideoFramerate->addItems( rates );

		// now create a graph to have more information
		AVFormatParameters par;

		tmpStr = ui.comboVideoNative->currentText();
		if (tmpStr != "") {
			tmpStr2 = tmpStr.section( 'x', 0, 0);
			par.width = tmpStr2.toInt();
			tmpStr2 = tmpStr.section( 'x', 1, 1);
			par.height = tmpStr2.toInt();
		}
		
		tmpStr = ui.comboVideoFramerate->currentText();
		if (tmpStr != "") {
			par.time_base.den = int (tmpStr.toDouble() * 1001);
			par.time_base.num = 1001;
		}

		std::string devutf8 = devStr.toUtf8();
		DShowVideoCaptureGraph vgraph( NULL, devutf8.c_str() );

		vgraph.CreateGraph( par );
		StringList filters_with_prop;
		IAMTVTuner *pTvTuner;
		IAMCrossbar	*pVideoCrossbar;
		vgraph.GetFiltersWithProps( filters_with_prop, &pTvTuner, &pVideoCrossbar );
		if (pTvTuner != NULL)
		{
			long AnalogVideoStandard;
			pTvTuner->get_AvailableTVFormats( &AnalogVideoStandard );
			pTvTuner->Release();

			if (DShowVideoCaptureGraph::FormatSupportsNTSC( AnalogVideoStandard ))
				if (!formats.contains( "NTSC" ))
					formats.push_back( "NTSC" );

			if (DShowVideoCaptureGraph::FormatSupportsPAL( AnalogVideoStandard ))
				if (!formats.contains( "PAL" ))
					formats.push_back( "PAL" );

			if (DShowVideoCaptureGraph::FormatSupportsSECAM( AnalogVideoStandard ))
				if (!formats.contains( "SECAM" ))
					formats.push_back( "SECAM" );
		}
		if (pVideoCrossbar != NULL)
		{
			// fill deviceinputs
			long outputpins, inputpins;
			pVideoCrossbar->get_PinCounts(&outputpins, &inputpins);
			for(long pin=0; pin<inputpins; ++pin) {
				long related;
				long phystype;

				if (FAILED(pVideoCrossbar->get_CrossbarPinInfo(TRUE, pin, &related, &phystype)))
					continue;

				tmpStr = QString::fromAscii( DShowCaptureGraph::GetPhysicalInput( phystype ) );
				inputs.push_back( tmpStr );
			}
			pVideoCrossbar->Release();
		}

		formats.sort();
		ui.comboVideoFormat->addItems( formats );
		ui.comboVideoInput->addItems( inputs );
	}
}

void captureUI::on_comboCaptureAudio_activated(QString)
{
	QString devStr = ui.comboCaptureAudio->currentText();

	if (devStr != audio_device)
	{
		std::wstring dev = devStr.toStdWString();
		audio_device = devStr;

		// update the Audio properties
		ui.comboAudioSample->clear();
		ui.comboAudioChannels->clear();
		ui.comboAudioInput->clear();

		StringList deviceinputs;
		std::vector<WAVEFORMATEX> devicecaps;
		DShowAudioCaptureGraph::GetDeviceCaps( dev, devicecaps, deviceinputs );

		QStringList samplerate, channels, inputs;
		QString tmpStr;
		size_t i;
		for (i=0; i<devicecaps.size(); i++)
		{
			// only frequencies supported by MP3
			switch (devicecaps[i].nSamplesPerSec)
			{
			case 8000:
			case 16000:
			case 32000:
			case 12000:
			case 24000:
			case 48000:
			case 11025:
			case 22050:
			case 44100:
				tmpStr.setNum( devicecaps[i].nSamplesPerSec );
				tmpStr += " Hz";
				if ( !samplerate.contains( tmpStr ) )
					samplerate.push_back( tmpStr );
				break;
			}

			tmpStr.setNum( devicecaps[i].nChannels );
			if ( !channels.contains( tmpStr ) )
				channels.push_back( tmpStr );
		}

		for (i=0; i<deviceinputs.size(); i++)
		{
			tmpStr = QString::fromStdWString( deviceinputs[i] );
			inputs.push_back( tmpStr );
		}

//		samplerate.sort();
		ui.comboAudioSample->addItems( samplerate );
//		channels.sort();
		ui.comboAudioChannels->addItems( channels );
		ui.comboAudioInput->addItems( inputs );
	}
}


void captureUI::on_buttonGetOuputDir_clicked()
{
	output_path = QFileDialog::getExistingDirectory( this, "Choose Output Directory", output_path );
	ui.editOutputDir->clear();
	ui.editOutputDir->insert( output_path );
}


void captureUI::on_buttonDebug_clicked()
{
	if (debug_wnd.isVisible())
		debug_wnd.hide();
	else
		debug_wnd.show();
}


void captureUI::closeEvent(QCloseEvent *e)
{
	debug_wnd.close();
}

void captureUI::DebugEvent( QEvent * event )
{
	if (event->type() != QDEBUG_MESSAGE_EVENT)
		return;

	debug_wnd.AddNewMessage( reinterpret_cast<QDebugMessageEvent*>(event)->GetString() );
}


void captureUI::on_buttonAudioConfigure_clicked()
{
	QString devStr = ui.comboCaptureAudio->currentText();
	std::wstring dev = devStr.toStdWString();

	DShowAudioCaptureGraph::Configure( dev.c_str(), winId() );
}

void captureUI::on_buttonVideoConfigure_clicked()
{
	QString tmpStr2,tmpStr = ui.comboCaptureVideo->currentText();
#if 0
	std::wstring dev = tmpStr.utf16();

	DShowVideoCaptureGraph::Configure( dev.c_str(), winId() );
#else
	std::string dev = tmpStr.toUtf8();
	DShowVideoCaptureGraph vgraph( NULL, dev.c_str() );

	AVFormatParameters par;

	tmpStr = ui.comboVideoNative->currentText();
	if (tmpStr != "") {
		tmpStr2 = tmpStr.section( 'x', 0, 0);
		par.width = tmpStr2.toInt();
		tmpStr2 = tmpStr.section( 'x', 1, 1);
		par.height = tmpStr2.toInt();
	}
	
	tmpStr = ui.comboVideoFramerate->currentText();
	if (tmpStr != "") {
		par.time_base.den = int (tmpStr.toDouble() * 1001);
		par.time_base.num = 1001;
	}

	vgraph.CreateGraph( par );
	StringList filters_with_prop;
	IAMTVTuner *pTvTuner;
	IAMCrossbar	*pVideoCrossbar;
	vgraph.GetFiltersWithProps( filters_with_prop, &pTvTuner, &pVideoCrossbar );

	if (pTvTuner != NULL)
		pTvTuner->Release();
	if (pVideoCrossbar != NULL)
		pVideoCrossbar->Release();
#endif
}
