/*
 * External debug handler for ffmpeg system
 * Copyright (c) 2005 DivX Inc
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <QCoreApplication>
#include <QCustomEvent>

#include "ffmpeg_dbg.h"

#define HAVE_AV_CONFIG_H
#include "avcodec.h"

QCoreApplication *pEventReceiver = NULL;

extern "C" {
/* the callback must be use a C call */
void av_log_win32(void *ptr, int level, const char* fmt, va_list vl)
{
	static char string[1024];
    static int print_prefix=1;
    AVClass* avc= ptr ? *(AVClass**)ptr : NULL;
    if(level>av_log_get_level())
        return;
    if(print_prefix && avc) {
		av_log(NULL, AV_LOG_DEBUG, "[%s @ %p]", avc->item_name(ptr), avc);
    }
        
    print_prefix= strstr(fmt, "\n") != NULL;
        
    vsprintf(string, fmt, vl);

	if (pEventReceiver != NULL)
		QCoreApplication::postEvent( pEventReceiver, new QDebugMessageEvent(string) );
}
};

void av_log_win32_init( QCoreApplication &receiver )
{
	pEventReceiver = &receiver;
	av_log_set_callback( av_log_win32 );
}



QDebugMessageEvent::QDebugMessageEvent( const char *str )
	:QEvent(QDEBUG_MESSAGE_EVENT)
{
	DebugMsgString = str;
}

const QString & QDebugMessageEvent::GetString() const
{
	return DebugMsgString;
}
