/*
 * AC3 DirectShow decoder (for win32 only)
 * Copyright (c) 2005 DivX, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
/**
 * @file SourceCallback.cpp
 * DShow AC3 input pin.
 */

#define AV_DONT_OVERRIDE_STDC
#include <streams.h>

#include "avcodec.h"

#include "Ac3SourceCallBack.h"


/**
 *	source class
**/

// initialize a sourcecallbackstream object so there is a pin
// 
Ac3SourceCallBack::Ac3SourceCallBack( WORD channels, DWORD SampleFreq )
: SourceCallBack( NAME("AC3 Source Filter"), channels, SampleFreq )
{
}

/**
 *	source output pin class
**/

Ac3SourceCallBackPin::Ac3SourceCallBackPin(Ac3SourceCallBack* pFilter, CCritSec* pLock, HRESULT* phr)
:	SourceCallBackPin( pFilter, pLock, phr)
{
}

Ac3SourceCallBackPin::~Ac3SourceCallBackPin()
{
	if (IsConnected())
		this->Disconnect();
}

HRESULT Ac3SourceCallBackPin::CheckMediaType(const CMediaType * pmt) 
{
	if (*pmt->Type() != MEDIATYPE_Audio || *pmt->Subtype() != MEDIASUBTYPE_DOLBY_AC3)
		return S_FALSE;
	return S_OK;
} 

HRESULT Ac3SourceCallBackPin::GetMediaType(int iPosition, CMediaType * pmt)
{
	if ( iPosition < 0 )
		return E_INVALIDARG;
	if ( iPosition != 0 )
		return VFW_S_NO_MORE_ITEMS;

	// the output pin force his mediatype to be used
	AM_MEDIA_TYPE * My = pmt;

	WAVEFORMATEX Fex;
	memset(&Fex, 0, sizeof Fex);
	Fex.nChannels      = static_cast<Ac3SourceCallBack*>(m_pFilter)->m_Channels;
	Fex.nSamplesPerSec = static_cast<Ac3SourceCallBack*>(m_pFilter)->m_SampleFreq;
	Fex.wBitsPerSample = 0;

	memset(My, 0, sizeof(AM_MEDIA_TYPE));
	My->majortype = MEDIATYPE_Audio;
	My->subtype = MEDIASUBTYPE_DOLBY_AC3;
	My->bFixedSizeSamples = TRUE;
	My->bTemporalCompression = FALSE;
	My->lSampleSize = 256 * 6 * 2;
	My->formattype = FORMAT_WaveFormatEx;
	My->pUnk = NULL;
	My->cbFormat = sizeof(WAVEFORMATEX);
	My->pbFormat = (unsigned char *)CoTaskMemAlloc(My->cbFormat);
	memcpy(My->pbFormat, &Fex, My->cbFormat);

	return S_OK;
}

HRESULT Ac3SourceCallBackPin::DecideBufferSize(IMemAllocator *pAlloc, 
	ALLOCATOR_PROPERTIES *pProperties) 
{
	HRESULT hr;
	ALLOCATOR_PROPERTIES Actual;

	pProperties->cBuffers = 1;
	pProperties->cbBuffer = AVCODEC_MAX_AUDIO_FRAME_SIZE;
	pProperties->cbAlign = 1;
	pProperties->cbPrefix = 0;

	hr = pAlloc->SetProperties(pProperties, &Actual);
	if (FAILED(hr)) {
		return hr;
	}

	// verify that the allocator is able to allocate what requested
	if (Actual.cbBuffer < pProperties->cbBuffer) {
		return E_FAIL;
	}

	return S_OK;
}
