/*
 * AC3 DirectShow decoder (for win32 only)
 * Copyright (c) 2005 DivX, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
/**
 * @file Ac3SourceCallback.h
 * DShow AC3 input pin.
 */

// void source filter to use the ac3 streaming buffer callback function
// this filter will hide our callback function call that will be 
// used by the smart and correlated output pin
//

#include "SourceCallback.h"

class Ac3SourceCallBack;

// output pin of the source filter, will use the callback function
//
class Ac3SourceCallBackPin : public SourceCallBackPin
{
public:
	Ac3SourceCallBackPin(Ac3SourceCallBack* pFilter, CCritSec* pLock, HRESULT* phr);
	~Ac3SourceCallBackPin();

	HRESULT CheckMediaType(const CMediaType * pmt);
	HRESULT GetMediaType(int iPosition, CMediaType * pmt);
	HRESULT DecideBufferSize(IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *ppropInputRequest);
};


// source filter
//
class Ac3SourceCallBack : public SourceCallBack
{
public:
	Ac3SourceCallBack( WORD channels, DWORD SampleFreq );

	CBasePin * GetPin(int n) {
		if (n != 0)
			return NULL;

		if (!m_pOutput) {
			HRESULT hr = S_OK;
			m_pOutput = new Ac3SourceCallBackPin(this, &m_Lock, &hr);
			if ( hr != S_OK )
			{
				delete m_pOutput;
				m_pOutput = NULL;
			}
		}
		return m_pOutput;
	}
/*
	bool Deliver( const BYTE *buf, int buf_size );
	void DeliverEndOfStream();
*/
};


