/*
 * DirectShow Audio Sink decoder (for win32 only)
 * Copyright (c) 2005 DivX, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
/**
 * @file AudioSinkCallBack.cpp
 * DShow PCM output pin.
 */

#define DISABLE_PTS
#define AV_DONT_OVERRIDE_STDC
#include <streams.h>

#include "AudioSinkCallBack.h"

#define SPEAKER_FRONT_LEFT              0x1
#define SPEAKER_FRONT_RIGHT             0x2
#define SPEAKER_FRONT_CENTER            0x4
#define SPEAKER_LOW_FREQUENCY           0x8
#define SPEAKER_BACK_LEFT               0x10
#define SPEAKER_BACK_RIGHT              0x20
#define SPEAKER_FRONT_LEFT_OF_CENTER    0x40
#define SPEAKER_FRONT_RIGHT_OF_CENTER   0x80
#define SPEAKER_BACK_CENTER             0x100
#define SPEAKER_SIDE_LEFT               0x200
#define SPEAKER_SIDE_RIGHT              0x400
#define SPEAKER_TOP_CENTER              0x800
#define SPEAKER_TOP_FRONT_LEFT          0x1000
#define SPEAKER_TOP_FRONT_CENTER        0x2000
#define SPEAKER_TOP_FRONT_RIGHT         0x4000
#define SPEAKER_TOP_BACK_LEFT           0x8000
#define SPEAKER_TOP_BACK_CENTER         0x10000
#define SPEAKER_TOP_BACK_RIGHT          0x20000

/**
 *	source class
**/

// initialize a sourcecallbackstream object so there is a pin
// 
AudioSinkCallBack::AudioSinkCallBack( AVCodecContext &avctx, DWORD SampleFreq )
 : SinkCallBack(avctx)
 , m_SampleFreq(SampleFreq)
{
	if (avctx.channels > 1)
		m_DownMixedChannels = 2;
	else
		m_DownMixedChannels = 1;
}

AudioSinkCallBack::~AudioSinkCallBack()
{
}


SinkCallBackPin *AudioSinkCallBack::CreateSinkPin(HRESULT & hr)
{
	return new AudioSinkCallBackPin(this, &m_Lock, &hr);
}

/**
 *	sink input pin class
**/

AudioSinkCallBackPin::AudioSinkCallBackPin(SinkCallBack* pFilter, CCritSec* pLock, HRESULT* phr)
:	SinkCallBackPin( pFilter, pLock, phr )
	,m_iTempOutputSize(0)
	,m_iTempPts(AV_NOPTS_VALUE)
	,m_pOutputBuf(m_pTempOutput)
	,m_piOutputSize(&m_iTempOutputSize)
	,m_TotalOutputSize(sizeof(m_pTempOutput))
	,m_pts(&m_iTempPts)
{
}

AudioSinkCallBackPin::~AudioSinkCallBackPin()
{
}

HRESULT AudioSinkCallBackPin::CheckMediaType(const CMediaType * pmt)
{
	if (*pmt->Type() != MEDIATYPE_Audio || (*pmt->Subtype() != MEDIASUBTYPE_PCM && *pmt->Subtype() != MEDIASUBTYPE_IEEE_FLOAT))
		return S_FALSE;
	if (pmt->formattype != FORMAT_WaveFormatEx)
		return S_FALSE;

	AudioSinkCallBack* pSink = static_cast<AudioSinkCallBack*>(m_pFilter);
	const WAVEFORMATEX *pFex = (WAVEFORMATEX *)pmt->pbFormat;
	if (pFex->nSamplesPerSec != pSink->m_SampleFreq)
		return S_FALSE;
	return S_OK;
}

HRESULT AudioSinkCallBackPin::GetMediaType(int iPosition, CMediaType * pmt)
{
	if ( iPosition < 0 )
		return E_INVALIDARG;
	if ( iPosition != 0 )
		return VFW_S_NO_MORE_ITEMS;

	// the output pin force his mediatypes to be used
	AudioSinkCallBack* pSink = static_cast<AudioSinkCallBack*>(m_pFilter);
	AM_MEDIA_TYPE * My = pmt;

#if 1
	WAVEFORMATEX Fex;
	memset(&Fex, 0, sizeof Fex);
	Fex.nChannels = pSink->avctx().channels;
	Fex.nSamplesPerSec = pSink->m_SampleFreq;
	Fex.wBitsPerSample = 16;
	Fex.nAvgBytesPerSec = (Fex.nChannels * Fex.nSamplesPerSec * Fex.wBitsPerSample) >> 3;
	Fex.nBlockAlign = 2*Fex.nChannels;
	Fex.wFormatTag = WAVE_FORMAT_PCM;

	memset(My, 0, sizeof(AM_MEDIA_TYPE));
	My->majortype = MEDIATYPE_Audio;
	My->subtype = MEDIASUBTYPE_PCM;
	My->bFixedSizeSamples = TRUE;
	My->bTemporalCompression = FALSE;
	My->lSampleSize = 1; // 256 * 6 * 2;
	My->formattype = FORMAT_WaveFormatEx;
	My->pUnk = NULL;
	My->cbFormat = sizeof(WAVEFORMATEX);
	My->pbFormat = (unsigned char *)CoTaskMemAlloc(My->cbFormat);
	memcpy(My->pbFormat, &Fex, My->cbFormat);
#else
	WAVEFORMATEXTENSIBLE Fex;
	memset(&Fex, 0, sizeof Fex);
	Fex.Format.nChannels = pSink->m_DownMixedChannels;
	Fex.Format.nSamplesPerSec = pSink->m_SampleFreq;
	Fex.Format.wBitsPerSample = 16;
	Fex.Format.nAvgBytesPerSec = (Fex.Format.nChannels * Fex.Format.nSamplesPerSec * Fex.Format.wBitsPerSample) >> 3;
	Fex.Format.nBlockAlign = 2*pSink->m_DownMixedChannels;
	Fex.Format.wFormatTag = WAVE_FORMAT_EXTENSIBLE;
	Fex.dwChannelMask = SPEAKER_FRONT_LEFT | SPEAKER_FRONT_RIGHT;
	Fex.SubFormat = MEDIASUBTYPE_PCM;

	memset(My, 0, sizeof(AM_MEDIA_TYPE));
	My->majortype = MEDIATYPE_Audio;
	My->subtype = MEDIASUBTYPE_PCM;
	My->bFixedSizeSamples = TRUE;
	My->bTemporalCompression = FALSE;
	My->lSampleSize = 1; // 256 * 6 * 2;
	My->formattype = FORMAT_WaveFormatEx;
	My->pUnk = NULL;
	My->cbFormat = sizeof(WAVE_FORMAT_EXTENSIBLE);
	My->pbFormat = (unsigned char *)CoTaskMemAlloc(My->cbFormat);
	memcpy(My->pbFormat, &Fex, My->cbFormat);
#endif

	return S_OK;
}

HRESULT AudioSinkCallBackPin::CompleteConnect(IPin *pReceivePin)
{
	AM_MEDIA_TYPE mt;
	ConnectionMediaType(&mt);

	if (mt.majortype != MEDIATYPE_Audio || (mt.subtype != MEDIASUBTYPE_PCM && mt.subtype != MEDIASUBTYPE_IEEE_FLOAT))
		return S_FALSE;
	if (mt.formattype != FORMAT_WaveFormatEx)
		return S_FALSE;

	AudioSinkCallBack* pSink = static_cast<AudioSinkCallBack*>(m_pFilter);
	const WAVEFORMATEX *pFex = (WAVEFORMATEX *)mt.pbFormat;

	if (pFex->nSamplesPerSec != pSink->m_SampleFreq || pFex->nChannels <= 0)
		return S_FALSE;

	if (!InitFromSampleType(mt))
		return E_FAIL;

	return CBasePin::CompleteConnect(pReceivePin);
}

bool AudioSinkCallBackPin::InitFromSampleType(const AM_MEDIA_TYPE & mt)
{
	AudioSinkCallBack* pSink = static_cast<AudioSinkCallBack*>(m_pFilter);
	const WAVEFORMATEX *pFex = (WAVEFORMATEX *)mt.pbFormat;

	// default values
	pSink->m_bFloatPCM = false;
	int mask = SPEAKER_FRONT_LEFT;
	pSink->m_InputChannelMask = 0;
	for (int channels = pFex->nChannels; channels > 0; channels--, mask<<=1)
		pSink->m_InputChannelMask |= mask;
	pSink->m_InputSinkChannels = pFex->nChannels;
	pSink->m_BitsPerSample = pFex->wBitsPerSample;
	pSink->m_InputAlign = pFex->nBlockAlign;
	pSink->m_factorLeft = pSink->m_factorRight = 1.0;

	if (pFex->wBitsPerSample == 8)
	{
		pSink->m_factorLeft  *= 0x100;
		pSink->m_factorRight *= 0x100;
	}
	else if (pFex->wBitsPerSample == 32)
	{
		pSink->m_factorLeft  /= 0x10000;
		pSink->m_factorRight /= 0x10000;
	}
	else if (pFex->wBitsPerSample != 16)
	{
		return false;
	}

	// get the channel mapping!
	if (pFex->wFormatTag == WAVE_FORMAT_EXTENSIBLE)
	{
		WAVEFORMATEXTENSIBLE *pFexSib = (WAVEFORMATEXTENSIBLE *) pFex;
		av_log(&pSink->avctx(), AV_LOG_DEBUG, "Channel Mask %x\n", pSink->m_InputChannelMask);

		if (pFexSib->SubFormat == MEDIASUBTYPE_PCM || pFexSib->SubFormat == MEDIASUBTYPE_IEEE_FLOAT)
		{
			// take in account energy levels, not just levels
			if (pFexSib->SubFormat == MEDIASUBTYPE_IEEE_FLOAT)
			{
				pSink->m_bFloatPCM = true;
				pSink->m_factorRight = pSink->m_factorLeft = 0x8000;
			}

			if (pFexSib->dwChannelMask > 0)
			{
				pSink->m_InputChannelMask = pFexSib->dwChannelMask;
			}
		}
	}
	else if (pFex->nChannels != 1 && pFex->nChannels != 2)
	{
		av_log(&pSink->avctx(), AV_LOG_ERROR, "Channel Mask was guessed: 0x%X\n", pSink->m_InputChannelMask);
	}


	if (pSink->m_InputChannelMask != SPEAKER_FRONT_LEFT|SPEAKER_FRONT_RIGHT &&
		pSink->m_InputChannelMask != SPEAKER_FRONT_LEFT)
	{
		int channelsOnLeft = 0, channelsOnRight = 0;

		pSink->m_InputSinkChannels = 0;

		// cancel the SPEAKER_BACK_CENTER channel, as with AC3Filter the audio is inverted
		//  thus the sum of a channel with the rear is 0... (2/1 - surround)
		if (pSink->m_InputChannelMask & SPEAKER_BACK_CENTER)
		{
			pSink->m_InputChannelMask &= ~SPEAKER_BACK_CENTER;
			av_log(&pSink->avctx(), AV_LOG_INFO, "Disable channel SPEAKER_BACK_CENTER as it might silence the front channels\n");
		}

		if (pSink->m_InputChannelMask & SPEAKER_FRONT_LEFT)
		{
			channelsOnLeft++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_FRONT_RIGHT)
		{
			channelsOnRight++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_FRONT_CENTER)
		{
			channelsOnLeft++;
			channelsOnRight++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_LOW_FREQUENCY)
		{
			channelsOnLeft++;
			channelsOnRight++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_BACK_LEFT)
		{
			channelsOnLeft++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_BACK_RIGHT)
		{
			channelsOnRight++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_FRONT_LEFT_OF_CENTER)
		{
			channelsOnLeft++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_FRONT_RIGHT_OF_CENTER)
		{
			channelsOnRight++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_BACK_CENTER)
		{
			channelsOnLeft++;
			channelsOnRight++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_SIDE_LEFT)
		{
			channelsOnLeft++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_SIDE_RIGHT)
		{
			channelsOnRight++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_TOP_CENTER)
		{
			channelsOnLeft++;
			channelsOnRight++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_TOP_FRONT_LEFT)
		{
			channelsOnLeft++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_TOP_FRONT_CENTER)
		{
			channelsOnLeft++;
			channelsOnRight++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_TOP_FRONT_RIGHT)
		{
			channelsOnRight++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_TOP_BACK_LEFT)
		{
			channelsOnLeft++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_TOP_BACK_CENTER)
		{
			channelsOnLeft++;
			channelsOnRight++;
			pSink->m_InputSinkChannels++;
		}
		if (pSink->m_InputChannelMask & SPEAKER_TOP_BACK_RIGHT)
		{
			channelsOnRight++;
			pSink->m_InputSinkChannels++;
		}

		if (channelsOnLeft > 1)
			pSink->m_factorLeft /= channelsOnLeft;
		if (channelsOnRight > 1)
			pSink->m_factorRight /= channelsOnRight;
	}

	return true;
}

void AudioSinkCallBackPin::SetOutput( void *outdata, int &outdata_size, int64_t *pts )
{
	CAutoLock( this->m_pLock );

	if ( m_iTempOutputSize != 0 && m_pOutputBuf == m_pTempOutput )
	{
		// keep the data read in the new buffer
		outdata_size = min(m_iTempOutputSize, outdata_size);
		memcpy( outdata, m_pTempOutput, outdata_size );
		m_iTempOutputSize -= outdata_size;
		assert(m_iTempOutputSize >= 0);
#ifndef DISABLE_PTS
		if (pts != NULL && m_pts != NULL) {
			*pts = *m_pts;
		}
#endif

		if (m_iTempOutputSize > 0) {
			AudioSinkCallBack* pSink = static_cast<AudioSinkCallBack*>(m_pFilter);
			memmove( m_pTempOutput, &m_pTempOutput[outdata_size], m_iTempOutputSize );
#ifndef DISABLE_PTS
			if (m_pts != NULL)
				*m_pts += outdata_size * AV_TIME_BASE / (pSink->m_SampleFreq * pSink->m_DownMixedChannels << 1);
#endif
		}
		m_piOutputSize = &m_iTempOutputSize;
		m_TotalOutputSize = sizeof(m_pTempOutput);
	}
	else
	{
		m_TotalOutputSize = outdata_size;
		outdata_size = 0;
		m_pOutputBuf = outdata;
		m_piOutputSize = &outdata_size;
		m_pts = pts;
	}
}

HRESULT AudioSinkCallBackPin::Receive(IMediaSample * pSample)
{
	CAutoLock( this->m_pLock );

	BYTE * pBuffer;
	LONG iSize = pSample->GetActualDataLength();

#ifndef DISABLE_PTS
	if (m_pts != NULL)
	{
		AudioSinkCallBack* pSink = static_cast<AudioSinkCallBack*>(m_pFilter);
		LONGLONG timeS,timeE;
		
		if (pSample->GetMediaTime(&timeS, &timeE) == S_OK)
			*m_pts = timeS * AV_TIME_BASE / pSink->m_SampleFreq;
	}
#endif

	AM_MEDIA_TYPE *pmt;
	if (pSample->GetMediaType(&pmt) == S_OK && pmt != NULL)
	{
		if (!InitFromSampleType(*pmt))
			return S_FALSE;
	}

	AudioSinkCallBack* pSink = static_cast<AudioSinkCallBack*>(m_pFilter);

	// downmix to mono/stereo 16 bits
	uint16_t sampleCount = iSize / pSink->m_InputAlign;
	int outputBufSize = sampleCount * 2 * pSink->m_DownMixedChannels;
	if ( *m_piOutputSize + outputBufSize > m_TotalOutputSize )
	{
		av_log(&pSink->avctx(), AV_LOG_DEBUG, "Sample too big (%d/%d) to fit our buffer\n", *m_piOutputSize, outputBufSize);
		// the sample is too big to be handled
		return IsConnected() ? S_OK : S_FALSE;
	}

	int16_t *pOutBuf = (int16_t*) ((int8_t*)m_pOutputBuf + *m_piOutputSize);
	pSample->GetPointer(&pBuffer);

	float sample_left, sample_right;
	for (uint16_t sample=0; sample<sampleCount; sample++, pOutBuf+=pSink->m_DownMixedChannels)
	{
		sample_left = sample_right = 0;
		const int8_t *samplePos = (int8_t*) &pBuffer[sample * pSink->m_InputAlign];

		if (pSink->m_InputChannelMask & SPEAKER_FRONT_LEFT)
		{
			pSink->EatSample(samplePos, sample_left);
		}
		if (pSink->m_InputChannelMask & SPEAKER_FRONT_RIGHT)
		{
			pSink->EatSample(samplePos, sample_right);
		}
		if (pSink->m_InputChannelMask & SPEAKER_FRONT_CENTER)
		{
			pSink->EatSample(samplePos, sample_left, sample_right);
		}
		if (pSink->m_InputChannelMask & SPEAKER_LOW_FREQUENCY)
		{
			pSink->EatSample(samplePos, sample_left, sample_right);
		}
		if (pSink->m_InputChannelMask & SPEAKER_BACK_LEFT)
		{
			pSink->EatSample(samplePos, sample_left);
		}
		if (pSink->m_InputChannelMask & SPEAKER_BACK_RIGHT)
		{
			pSink->EatSample(samplePos, sample_right);
		}
		if (pSink->m_InputChannelMask & SPEAKER_FRONT_LEFT_OF_CENTER)
		{
			pSink->EatSample(samplePos, sample_left);
		}
		if (pSink->m_InputChannelMask & SPEAKER_FRONT_RIGHT_OF_CENTER)
		{
			pSink->EatSample(samplePos, sample_right);
		}
		if (pSink->m_InputChannelMask & SPEAKER_BACK_CENTER)
		{
			pSink->EatSample(samplePos, sample_left, sample_right);
		}
		if (pSink->m_InputChannelMask & SPEAKER_SIDE_LEFT)
		{
			pSink->EatSample(samplePos, sample_left);
		}
		if (pSink->m_InputChannelMask & SPEAKER_SIDE_RIGHT)
		{
			pSink->EatSample(samplePos, sample_right);
		}
		if (pSink->m_InputChannelMask & SPEAKER_TOP_CENTER)
		{
			pSink->EatSample(samplePos, sample_left, sample_right);
		}
		if (pSink->m_InputChannelMask & SPEAKER_TOP_FRONT_LEFT)
		{
			pSink->EatSample(samplePos, sample_left);
		}
		if (pSink->m_InputChannelMask & SPEAKER_TOP_FRONT_CENTER)
		{
			pSink->EatSample(samplePos, sample_left, sample_right);
		}
		if (pSink->m_InputChannelMask & SPEAKER_TOP_FRONT_RIGHT)
		{
			pSink->EatSample(samplePos, sample_right);
		}
		if (pSink->m_InputChannelMask & SPEAKER_TOP_BACK_LEFT)
		{
			pSink->EatSample(samplePos, sample_left);
		}
		if (pSink->m_InputChannelMask & SPEAKER_TOP_BACK_CENTER)
		{
			pSink->EatSample(samplePos, sample_left, sample_right);
		}
		if (pSink->m_InputChannelMask & SPEAKER_TOP_BACK_RIGHT)
		{
			pSink->EatSample(samplePos, sample_right);
		}

		sample_left  *= pSink->m_factorLeft;
		if (pSink->m_InputSinkChannels > 1)
			sample_right *= pSink->m_factorRight;
		else
			sample_right = sample_left;

		assert(pSink->m_DownMixedChannels == 1 || pSink->m_DownMixedChannels == 2);
		pOutBuf[0] = sample_left;
		if (pSink->m_DownMixedChannels > 1)
			pOutBuf[1] = sample_right;
	}
	*m_piOutputSize += outputBufSize;

	m_pOutputBuf = m_pTempOutput;
	m_piOutputSize = &m_iTempOutputSize;
	m_TotalOutputSize = sizeof(m_pTempOutput);
	m_pts = &m_iTempPts;

	return S_OK;
}

void AudioSinkCallBack::EatSample(const int8_t * &InputBuffer, float & outChannel) const
{
	float sample = ExtractSample(InputBuffer);

	outChannel += sample;
}

void AudioSinkCallBack::EatSample(const int8_t * &InputBuffer, float & outChannel1, float & outChannel2) const
{
	float sample = ExtractSample(InputBuffer);

	outChannel1 += sample;
	outChannel2 += sample;
}

float AudioSinkCallBack::ExtractSample(const int8_t * &InputBuffer) const
{
	float result = 0;
	if (m_bFloatPCM)
	{
		result = *(float*)InputBuffer;
	}
	else
	{
		if (m_BitsPerSample == 8)
		{
			result = *(int8_t*)InputBuffer;
		}
		else if (m_BitsPerSample == 16)
		{
			result = *(int16_t*)InputBuffer;
		}
		else if (m_BitsPerSample == 32)
		{
			result = *(int32_t*)InputBuffer;
		}
		else
		{
			av_log(&m_avctx, AV_LOG_ERROR, "BitsPerSample %d not supported\n", m_BitsPerSample);
		}
	}

	InputBuffer += (m_BitsPerSample / 8);
	if (m_BitsPerSample % 8 != 0)
		InputBuffer++;

	return result;
}
