/*
 * DirectShow Audio Sink decoder (for win32 only)
 * Copyright (c) 2005 DivX, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
/**
 * @file AudioSinkCallBack.h
 * DShow PCM output pin.
 */

#include <cstdio>

#include "avcodec.h"
#include "SinkCallBack.h"

// output pin of the source filter, will use the callback function
//
class AudioSinkCallBackPin: public SinkCallBackPin
{
public:
	AudioSinkCallBackPin(SinkCallBack* pFilter, CCritSec* pLock, HRESULT* phr);
	~AudioSinkCallBackPin();

    HRESULT GetMediaType(int iPosition, CMediaType * pmt);
    HRESULT CheckMediaType(const CMediaType* pmt);
    // implements the IMemInputPin::Receive method
    // receive the decoded audio sample 
    STDMETHODIMP Receive(IMediaSample* pSample);
	void SetOutput( void *outdata, int &outdata_size, int64_t *pts = NULL );
	STDMETHODIMP ReceiveCanBlock()
	{
		return S_FALSE;
	}
	void WaitOutput() {}
	HRESULT CompleteConnect(IPin *pReceivePin);

private:
	int   m_iTempOutputSize;
	BYTE  m_pTempOutput[AVCODEC_MAX_AUDIO_FRAME_SIZE];
	int64_t m_iTempPts;
	void *m_pOutputBuf;
	int  *m_piOutputSize;
	int  m_TotalOutputSize;
	int64_t *m_pts;

	bool InitFromSampleType(const AM_MEDIA_TYPE & mt);
};


// source filter
//
class AudioSinkCallBack : public SinkCallBack
{
public:
	AudioSinkCallBack( AVCodecContext & avctx, DWORD SampleFreq );
	~AudioSinkCallBack();

	DWORD m_SampleFreq;
	int m_InputSinkChannels;
	int m_InputAlign;
	DWORD m_InputChannelMask;
	float m_factorLeft, m_factorRight;
	int m_BitsPerSample;
	bool m_bFloatPCM;
	int m_DownMixedChannels;

	inline void EatSample(const int8_t * &InputBuffer, float & outChannel) const;
	inline void EatSample(const int8_t * &InputBuffer, float & outChannel1, float & outChannel2) const;

private:
	SinkCallBackPin *CreateSinkPin(HRESULT & hr);
	inline float ExtractSample(const int8_t * &InputBuffer) const;
};


