/*
 * Generic DirectShow decoder (for win32 only)
 * Copyright (c) 2006 DivX, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
/**
 * @file DShowDecoder.h
 * DShow utilities.
 */


#ifndef _AV_DSHOW_DECODER_H_
#define _AV_DSHOW_DECODER_H_

#include "SourceCallBack.h"
#include "SinkCallBack.h"

#include <vector>
typedef std::vector<std::wstring> StringList;

class DShowDecoder
{
public:
    DShowDecoder()
        :m_bGraphInit(false)
        ,m_oGraph(NULL)
        ,m_oGraphControl(NULL)
        ,pSource(NULL)
        ,pSink(NULL)
        ,pSourceFilter(NULL)
        ,pSinkFilter(NULL)
    {
    }
    virtual ~DShowDecoder();

    bool m_bGraphInit;

    bool InitGraph( AVCodecContext *avctx, const uint8_t *buf, int buf_size );

    bool Deliver( const uint8_t *buf, int buf_size );
    void SetOutput( void *outdata, int &outdata_size, int64_t *pts = NULL )
    {
        pSink->SetOutput( outdata, outdata_size, pts );
    }

    bool GivePinStrName(IBaseFilter * pDecoderFilter, LPWSTR& PinOneIdName, LPWSTR& PinTwoIdName);

    void GraphInfo(AVCodecContext *avctx);
    void GetFilterName(char *string, unsigned int buf_size);
    void DeliverEndOfStream();

protected:
    bool ConnectGraph( IGraphBuilder *m_oGraph, IBaseFilter *pSourceFilter, IBaseFilter *pDecoderFilter, IBaseFilter *pSinkFilter );
    bool ConnectGraph( IGraphBuilder *m_oGraph, IBaseFilter *pSourceFilter, IBaseFilter *pSinkFilter );

    virtual SourceCallBack *GetSource(AVCodecContext *avctx, const uint8_t *buf, int buf_size) = 0;
    virtual SinkCallBack *GetSink(AVCodecContext *avctx, const uint8_t *buf, int buf_size) = 0;
	virtual const std::vector<whitelist> & GetWhiteList() const = 0;

    IGraphBuilder *m_oGraph;
    IMediaControl *m_oGraphControl;
    SourceCallBack *pSource;
    SinkCallBack *pSink;
    IBaseFilter *pSourceFilter, *pSinkFilter;
    DWORD dwRotId;
};

#endif // _AV_DSHOW_DECODER_H_
