/*
 * DTS DirectShow decoder (for win32 only)
 * Copyright (c) 2005 DivX, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
/**
 * @file SourceCallback.cpp
 * DShow DTS input pin.
 */

#define AV_DONT_OVERRIDE_STDC
#include <streams.h>

#include "avcodec.h"

#include "DtsSourceCallBack.h"


/**
 *	source class
**/

// initialize a sourcecallbackstream object so there is a pin
// 
DtsSourceCallBack::DtsSourceCallBack( WORD channels, DWORD SampleFreq, const void *codec_data, int codec_data_size )
: SourceCallBack( NAME("DTS Source Filter"), channels, SampleFreq )
,m_CodecData(codec_data)
,m_CodecDataSize(codec_data_size)
{
}

/**
 *	source output pin class
**/

DtsSourceCallBackPin::DtsSourceCallBackPin(DtsSourceCallBack* pFilter, CCritSec* pLock, HRESULT* phr)
:	SourceCallBackPin( pFilter, pLock, phr)
{
}

DtsSourceCallBackPin::~DtsSourceCallBackPin()
{
	if (IsConnected())
		this->Disconnect();
}

HRESULT DtsSourceCallBackPin::CheckMediaType(const CMediaType * pmt) 
{
#if 1
	if (*pmt->Type() != MEDIATYPE_Audio || *pmt->Subtype() != FOURCCMap(0x2001))
		return S_FALSE;
	if (pmt->formattype != FORMAT_WaveFormatEx)
		return S_FALSE;

	const WAVEFORMATEX *pFex = (WAVEFORMATEX *)pmt->pbFormat;
	if (pFex->wFormatTag != 0x2001)
		return S_FALSE;
#else
	if (*pmt->Type() != MEDIATYPE_Audio || *pmt->Subtype() != MEDIASUBTYPE_DTS)
		return S_FALSE;
#endif
	return S_OK;
} 

HRESULT DtsSourceCallBackPin::GetMediaType(int iPosition, CMediaType * pmt)
{
	if ( iPosition < 0 )
		return E_INVALIDARG;
	if ( iPosition > 1 )
		return VFW_S_NO_MORE_ITEMS;

	// the output pin force his mediatype to be used
	AM_MEDIA_TYPE * My = pmt;
	DtsSourceCallBack *pSource = static_cast<DtsSourceCallBack*>(m_pFilter);

	WAVEFORMATEX *pFex = (WAVEFORMATEX *)new char[sizeof(WAVEFORMATEX)];
	memset(pFex, 0, sizeof(WAVEFORMATEX));
	pFex->nChannels      = pSource->m_Channels;
	pFex->nSamplesPerSec = pSource->m_SampleFreq;
	pFex->wBitsPerSample = 0;
	pFex->wFormatTag = 0x2001;

	memset(My, 0, sizeof(AM_MEDIA_TYPE));
	My->majortype = MEDIATYPE_Audio;
	My->subtype = FOURCCMap(pFex->wFormatTag);
//	My->subtype = MEDIASUBTYPE_DTS;
	My->bFixedSizeSamples = FALSE;
	My->bTemporalCompression = FALSE;
	My->lSampleSize = 256 * 6 * 2;
	My->formattype = FORMAT_WaveFormatEx;
	My->pUnk = NULL;
	My->cbFormat = sizeof(WAVEFORMATEX) + pFex->cbSize;
	My->pbFormat = (unsigned char *)CoTaskMemAlloc(My->cbFormat);
	memcpy(My->pbFormat, pFex, My->cbFormat);
	delete[] (char *)pFex;

	return S_OK;
}

HRESULT DtsSourceCallBackPin::DecideBufferSize(IMemAllocator *pAlloc, 
	ALLOCATOR_PROPERTIES *pProperties) 
{
	HRESULT hr;
	ALLOCATOR_PROPERTIES Actual;

	pProperties->cBuffers = 1;
	pProperties->cbBuffer = AVCODEC_MAX_AUDIO_FRAME_SIZE;
	pProperties->cbAlign = 1;
	pProperties->cbPrefix = 0;

	hr = pAlloc->SetProperties(pProperties, &Actual);
	if (FAILED(hr)) {
		return hr;
	}

	// verify that the allocator is able to allocate what requested
	if (Actual.cbBuffer < pProperties->cbBuffer) {
		return E_FAIL;
	}

	return S_OK;
}
