/*
 * RealVideo DirectShow decoder, using Gabest's filter (for win32 only)
 * Copyright (c) 2006 DivX, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
/**
 * @file RealVideoDecoder.h
 * DShow utilities.
 */


#ifndef _RV_DSHOW_DECODER_H_
#define _RV_DSHOW_DECODER_H_

#include <vector>
typedef std::vector<std::wstring> StringList;

#include "RealVideoSourceCallback.h"
#include "DShowDecoder.h"

class RealVideoDecoder : public DShowDecoder
{
public:
    RealVideoDecoder(AVCodecContext *avcotx)
        :frame_len(0)
        ,frame_buffer(NULL)
        ,m_avctx(avcotx)
    {
        frame.data[0] = NULL;
    }
    ~RealVideoDecoder();

    bool InitGraph( AVCodecContext *avctx, const uint8_t *buf, int buf_size );

    AVFrame  frame;
    int      frame_len;
    uint8_t  *frame_buffer;
protected:
	std::vector<whitelist> WhiteList;

    virtual SourceCallBack *GetSource(AVCodecContext *avctx, const uint8_t *buf, int buf_size);
    virtual SinkCallBack *GetSink(AVCodecContext *avctx, const uint8_t *buf, int buf_size);
	virtual const std::vector<whitelist> & GetWhiteList() const { return WhiteList; }

	AVCodecContext *m_avctx;
};

#endif // _RV_DSHOW_DECODER_H_
