/*
 * RealVideo DirectShow decoder, using Gabest's filter (for win32 only)
 * Copyright (c) 2006 DivX, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
/**
 * @file RealVideoSourceCallback.h
 * DShow RealVideo input pin.
 */

// void source filter to use the aac streaming buffer callback function
// this filter will hide our callback function call that will be 
// used by the smart and correlated output pin
//

#include "SourceCallback.h"

class RealVideoSourceCallback;

// output pin of the source filter, will use the callback function
//
class RealVideoSourceCallbackPin : public SourceCallBackPin
{
public:
	RealVideoSourceCallbackPin(RealVideoSourceCallback* pFilter, CCritSec* pLock, HRESULT* phr);

	HRESULT CheckMediaType(const CMediaType * pmt);
	HRESULT GetMediaType(int iPosition, CMediaType * pmt);
	HRESULT DecideBufferSize(IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *ppropInputRequest);
};


// source filter
//
class RealVideoSourceCallback : public SourceCallBack
{
public:
	RealVideoSourceCallback( uint16_t width, uint16_t height, int codec, const void *codec_data, int codec_data_size );

	CBasePin * GetPin(int n) {
		if (n != 0)
			return NULL;

		if (!m_pOutput) {
			HRESULT hr = S_OK;
			m_pOutput = new RealVideoSourceCallbackPin(this, &m_Lock, &hr);
			if ( hr != S_OK )
			{
				delete m_pOutput;
				m_pOutput = NULL;
			}
		}
		return m_pOutput;
	}

	const void *m_CodecData;
	size_t      m_CodecDataSize;
	uint16_t    m_width;
	uint16_t    m_height;
	int         rvcodec;

	unsigned long GetCodecGUID() const;
};


