/*
 * DirectShow Video Sink decoder (for win32 only)
 * Copyright (c) 2005 DivX, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
/**
 * @file SinkCallBack.h
 * DShow output pin.
 */

#ifndef _AV_SINK_CALLBACK_
#define _AV_SINK_CALLBACK_

#define AV_DONT_OVERRIDE_STDC
#include "common.h"
#include <streams.h>

struct AVCodecContext;
class SinkCallBack;

// output pin of the source filter, will use the callback function
//
class SinkCallBackPin: public CBaseInputPin
{
public:
	SinkCallBackPin(SinkCallBack* pFilter, CCritSec* pLock, HRESULT* phr);
	virtual ~SinkCallBackPin();

    virtual HRESULT GetMediaType(int iPosition, CMediaType * pmt) =0;
    virtual HRESULT CheckMediaType(const CMediaType* pmt) = 0;
    // implements the IMemInputPin::Receive method
    // receive the decoded audio sample 
    virtual STDMETHODIMP Receive(IMediaSample* pSample) = 0;
	virtual void SetOutput( void *outdata, int &outdata_size, int64_t *pts = NULL ) = 0;
	virtual void WaitOutput() = 0;
};

// source filter
//
class SinkCallBack : public CBaseFilter
{
public:
	SinkCallBack(AVCodecContext & avctx);
	virtual ~SinkCallBack();

	int GetPinCount() {
		return 1;
	}
	CBasePin * GetPin(int n) {
		if (n != 0)
			return NULL;

		if (!m_pInput) {
			HRESULT hr = S_OK;
			m_pInput = CreateSinkPin(hr);
			if ( hr != S_OK )
			{
				delete m_pInput;
				m_pInput = NULL;
			}
		}
		return m_pInput;
	}
	void SetOutput( void *outdata, int &outdata_size, int64_t *pts = NULL )
	{
		m_pInput->SetOutput( outdata, outdata_size, pts );
	}
	void WaitOutput()
	{
		m_pInput->WaitOutput();
	}

	AVCodecContext &avctx()
	{
		return m_avctx;
	}

protected:
	virtual SinkCallBackPin *CreateSinkPin(HRESULT & hr) = 0;

	CCritSec m_Lock;
	SinkCallBackPin *m_pInput;
	AVCodecContext &m_avctx;
};

#endif // _AV_SINK_CALLBACK_
