/*
 * DirectShow decoder (for win32 only)
 * Copyright (c) 2005 DivX, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
/**
 * @file SourceCallback.cpp
 * DShow input pin.
 */


#include <streams.h>

#include "SourceCallBack.h"


SourceCallBackPin::~SourceCallBackPin()
{
	if (IsConnected())
		this->Disconnect();
}

/**
 *	source class
**/

// initialize a sourcecallbackstream object so there is a pin
// 
SourceCallBack::SourceCallBack( const TCHAR *FilterName, WORD channels, DWORD SampleFreq )
: CBaseFilter(
    FilterName,
    NULL,
    &m_Lock,
    GUID_NULL)
  ,m_pOutput(NULL)
  ,m_Channels(channels)
  ,m_SampleFreq(SampleFreq)
{
}

SourceCallBack::~SourceCallBack()
{
	if ( m_pOutput != NULL )
	{
		delete m_pOutput;
	}
}

/**
 *	source output pin class
**/

SourceCallBackPin::SourceCallBackPin(SourceCallBack* pFilter, CCritSec* pLock, HRESULT* phr)
:	CBaseOutputPin(NAME("Output"), pFilter, pLock, phr, L"Output")
{
}

bool SourceCallBack::Deliver( const BYTE *buf, int buf_size )
{
	IMediaSample *pSample;
	BYTE *pBuffer;
	bool result = false;
	REFERENCE_TIME rtStart = 0, rtStop = 0;
	if (FAILED(m_pOutput->GetDeliveryBuffer( &pSample, NULL, NULL, 0)))
		goto done_deliver;

	long bSize = pSample->GetSize();

	if (bSize < buf_size)
		goto done_deliver;

	if (FAILED(pSample->GetPointer( &pBuffer )))
		goto done_deliver;

	memcpy( pBuffer, buf, buf_size );
	pSample->SetActualDataLength( buf_size );

	pSample->SetTime(&rtStart, &rtStop);

	if (FAILED(m_pOutput->Deliver( pSample )))
		goto done_deliver;

	pSample->Release();

	result = true;
done_deliver:

	return result;
}

void SourceCallBack::DeliverEndOfStream()
{
	m_pOutput->DeliverEndOfStream();
}

