/*
 * DirectShow decoder (for win32 only)
 * Copyright (c) 2005 DivX, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
/**
 * @file SourceCallback.h
 * DShow input pin.
 */

// void source filter to use the ac3 streaming buffer callback function
// this filter will hide our callback function call that will be 
// used by the smart and correlated output pin
//

#ifndef _AV_SOURCE_CALLBACK_
#define _AV_SOURCE_CALLBACK_

class SourceCallBack;

// output pin of the source filter, will use the callback function
//
class SourceCallBackPin : public CBaseOutputPin
{
public:
	SourceCallBackPin(SourceCallBack* pFilter, CCritSec* pLock, HRESULT* phr);
	virtual ~SourceCallBackPin();

	virtual HRESULT CheckMediaType(const CMediaType * pmt) = 0;
	virtual HRESULT GetMediaType(int iPosition, CMediaType * pmt) = 0;
	virtual HRESULT DecideBufferSize(IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *ppropInputRequest) = 0;
};


// void source filter to use the ac3 streaming buffer callback function
// this filter will hide our callback function call that will be 
// used by the smart and correlated output pin
//

// source filter
//
class SourceCallBack : public CBaseFilter
{
public:
	SourceCallBack( const TCHAR *FilterName, WORD channels, DWORD SampleFreq );
	virtual ~SourceCallBack();

	int GetPinCount() {
		return 1;
	}
	virtual CBasePin * GetPin(int n) = 0;
	bool Deliver( const BYTE *buf, int buf_size );
	void DeliverEndOfStream();

	WORD m_Channels;
	DWORD m_SampleFreq;

protected:
	CCritSec m_Lock;
	SourceCallBackPin * m_pOutput;
};

#endif // _AV_SOURCE_CALLBACK_
