/*
 * DirectShow capture io for ffmpeg system
 * Copyright (c) 2005 DivX Inc
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */

#ifndef _DSHOW_VIDEO_CAP_GRAPH_H_
#define _DSHOW_VIDEO_CAP_GRAPH_H_

#include "capture.h"

class VideoSinkCallBack;

class DShowVideoCaptureGraph : public DShowCaptureGraph {
public:
	DShowVideoCaptureGraph(AVFormatContext *avctx, const char *device);
	~DShowVideoCaptureGraph();

	int ReadFrame(AVPacket *pkt);

	bool CreateGraph( AVFormatParameters & avctx );

	bool FillCodec(AVFormatParameters & avfctx);

	size_t GetFiltersWithProps( StringList & out_list, IAMTVTuner ** pTvTuner, IAMCrossbar ** pVideoCrossbar );

	static size_t GetDeviceList( StringList & out_list, AVFormatContext *avctx = NULL );
	static void GetDeviceCaps( const std::wstring & device, std::vector<VIDEOINFOHEADER> & out_list, AVFormatContext *avctx = NULL );
	static void Configure( const wchar_t *device, HWND hwParent = NULL );

	static bool FormatSupportsNTSC(ULONG format);
	static bool FormatSupportsPAL(ULONG format);
	static bool FormatSupportsSECAM(ULONG format);

	AVCodecContext m_avcctx;
protected:
	HRESULT FindCaptureDevice(IBaseFilter ** ppSrcFilter);
	char *m_device_input;
};

#endif // _DSHOW_VIDEO_CAP_GRAPH_H_
