/*
 * DirectShow Video Sink decoder (for win32 only)
 * Copyright (c) 2005 DivX, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
/**
 * @file VideoSinkCallBack.h
 * DShow YUV output pin.
 */

#include <cstdio>

//#include "avcodec.h"
#include "avformat.h"
#include "SinkCallBack.h"

struct av_GUID_pix_fmt {
	GUID uid;
	PixelFormat pix_fmt;
	int bits_p_px;
};

extern av_GUID_pix_fmt av_guid_pixfmt_table[8];

// output pin of the source filter, will use the callback function
//
class VideoSinkCallBackPin: public SinkCallBackPin
{
public:
	VideoSinkCallBackPin(SinkCallBack* pFilter, CCritSec* pLock, HRESULT* phr);
	~VideoSinkCallBackPin();

    HRESULT GetMediaType(int iPosition, CMediaType * pmt);
    HRESULT CheckMediaType(const CMediaType* pmt);
    //HRESULT BreakConnect(void);
    // implements the IMemInputPin::Receive method
    // receive the decoded audio sample 
    STDMETHODIMP Receive(IMediaSample* pSample);
	void SetOutput( void *outdata, int &outdata_size, int64_t *pts = NULL );
	STDMETHODIMP ReceiveCanBlock()
	{
		return S_OK;
	}
	HRESULT SetMediaType(const CMediaType *pmt);
	void WaitOutput();

	static PixelFormat VideoFormatSubtype(GUID & uid);

private:
	HANDLE m_hMemoryGiven, m_hMemoryFilled;
	void *m_pOutputBuf;
	int  *m_piOutputSize;
	LONGLONG m_FrameTime; // equivalent to DTS/PTS
	int64_t *m_pts;
};


// source filter
//
class VideoSinkCallBack : public SinkCallBack
{
public:
	VideoSinkCallBack( AVCodecContext & avctx );
	~VideoSinkCallBack();

	int m_Width, m_Height;
	LONGLONG m_FrameDuration;

protected:
	SinkCallBackPin *CreateSinkPin(HRESULT & hr);
};
