/*
 * AAC DirectShow decoder (for win32 only)
 * Copyright (c) 2005 DivX, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
/**
 * @file aac_dshow.cpp
 * AAC DShow entry point.
 */

#include <string>

#include <streams.h>

#include "ffconfig.h"
#include "common.h"
#include "avformat.h"

#if defined(CONFIG_WIN32)

#include "FilterUtil.h"
#include "AacSourceCallBack.h"
#include "AudioSinkCallBack.h"
#include "AacDecoder.h"

std::vector<whitelist> AacDecoder::WhiteList;

SourceCallBack *AacDecoder::GetSource(AVCodecContext *avctx, const uint8_t *buf, int buf_size)
{
	return new AacSourceCallBack( avctx->channels, avctx->sample_rate, avctx->extradata, avctx->extradata_size );
}

SinkCallBack *AacDecoder::GetSink(AVCodecContext *avctx, const uint8_t *buf, int buf_size)
{
	assert(avctx != NULL);
	return new AudioSinkCallBack( *avctx, avctx->sample_rate );
}

#define _T(x) x

extern "C" {

#include "avcodec.h"

static int aac_decode_init(AVCodecContext *avctx)
{
	avctx->sample_fmt= SAMPLE_FMT_S16;
	avctx->priv_data = new AacDecoder();

	return avctx->priv_data ? 0 : -1;
}

static int aac_decode_end(AVCodecContext *avctx)
{
	AacDecoder *pDecoder = (AacDecoder *)avctx->priv_data;
	delete pDecoder;
	avctx->priv_data = NULL;	
	return 0;
}

static int aac_decode_frame(AVCodecContext *avctx, void *outdata, int *outdata_size,
                uint8_t *buf, int buf_size)
{
	AacDecoder *pDecoder = (AacDecoder *)avctx->priv_data;

	if (!pDecoder->m_bGraphInit)
	{
        if (!pDecoder->InitGraph( avctx, buf, buf_size ))
			return -1;

		pDecoder->GraphInfo( avctx );
	}

	*outdata_size = AVCODEC_MAX_AUDIO_FRAME_SIZE; // known size for all audio codec
	pDecoder->SetOutput( outdata, *outdata_size );

	if ( buf_size == 0 )
		pDecoder->DeliverEndOfStream();

	if (pDecoder->Deliver( buf, buf_size ))
		return buf_size;

	return -1;
}

static const char *aac_item_name(void*)
{
	return "aac_dshow";
}
static AVClass aac_dshow_class = {"aac_dshow", aac_item_name, NULL};

const char *aac_dshow_filter_string(void)
{
	static char *filter_string = NULL;
	if (filter_string == NULL)
	{		
		AacDecoder *decoder = new AacDecoder();
		uint8_t aac_buf[5]= {0x13 ,0x10 ,0x56 ,0xe5 ,0x98};
		AVCodecContext ctx;
		ctx.av_class = &aac_dshow_class;
		ctx.channels = 2;
		ctx.sample_rate = 44100;
		ctx.extradata = aac_buf;
		ctx.extradata_size = 5;

		if (!decoder->InitGraph( &ctx, aac_buf, 5 ))
			goto clean_exit;
		filter_string = (char*) av_malloc(256);
		decoder->GetFilterName( filter_string, 256 );
clean_exit:
		delete decoder;		
	}
	return filter_string;
}

int av_aac_dshow_whitelist(int argc, const whitelist *whitelists)
{
	int i;

	AacDecoder::WhiteList.clear();

	for (i=0; i<argc; i++)
	{
		AacDecoder::WhiteList.push_back(whitelists[i]);
	}

	return 0;
}

typedef AacDecoder *AacDecoderPrivate;

AVCodec aac_dshow_decoder = {
    "aac_dshow",
    CODEC_TYPE_AUDIO,
    CODEC_ID_AAC,
    sizeof(AacDecoderPrivate),
    aac_decode_init,
    NULL,
    aac_decode_end,
    aac_decode_frame,
    /*.capabilities= */CODEC_CAP_DELAY,
};

};

#endif // CONFIG_WIN32
