/*
 *  Copyright (C) 2005, DivX Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
*/
/**
    \file aacframe.cpp
*/

#include <cassert>

#include "aacframe.h"

const uint32_t AAC_Frame::SamplingRates[12] =
    //  00,   01,   10, 11
    {96000, 88200, 64000, 48000, 44100, 32000, 24000, 22050, 16000, 12000, 11025,  8000};

const int16_t AAC_Frame::Bitrates[20] =
   { 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384, 448, 512, 576, 640, -1};

AAC_Frame::AAC_Frame()
 :bValid(false)
{}

/**
    \todo compute frame size
*/
bool AAC_Frame::Eval(const uint8_t *Header, size_t HeadSize)
{
	if (HeadSize < 5) {
		bValid = false;
		return false;
	}

    bValid = (Header[0] == 0xFF) && ((Header[1] & 0xF0) == 0xF0);

    myBinaryValue[0] = Header[0];
    myBinaryValue[1] = Header[1];
    myBinaryValue[2] = Header[2];
    myBinaryValue[3] = Header[3];
    myBinaryValue[4] = Header[4];

//    big_uint16 bu16;
//    bu16.Eval(&Header[2]);
//    myCrc1 = bu16;
    myCrc1 = Header[2] << 8 | Header[3];
    
    // sampling rate
    mySamplingRate = SamplingRates[(Header[4] >> 6) & 0x03];

	if (mySamplingRate == 0)
		bValid = false;

    return bValid;
}

/*
uint16_t AAC_Frame::SampleSize() const
{
    return 0; // unsupported format
}
*/

/**
    \todo simplify with more general cases
*/
uint16_t AAC_Frame::DataSize() const
{
    assert(bValid);

    return myFrameSize;
}

bool AAC_Frame::Compatible(const AAC_Frame & aFrame) const
{
    if (!aFrame.bValid)
	return false;

    if (mySamplingRate != aFrame.mySamplingRate)
	return false;

  return true;
}

uint16_t AAC_Frame::Channels() const
{
	uint16_t result = myLfeChannel ? 1: 0;
    switch (myChannels)
    {
	case AAC_1_AND_1:
		result += 2;
		break;
	case AAC_CENTER:
		result += 1;
		break;
	case AAC_LEFT_RIGHT:
		result += 2;
		break;
	case AAC_LEFT_CENTER_RIGHT:
		result += 3;
		break;
	case AAC_LEFT_RIGHT_SURROUND:
		result += 3;
		break;
	case AAC_LEFT_CENTER_RIGHT_SURROUND:
		result += 4;
		break;
	case AAC_LEFT_SURROUND_RIGHT_SURROUND:
		result += 4;
		break;
	case AAC_CENTER_LEFT_SURROUND_RIGHT_SURROUND:
		result += 5;
		break;
    }
    return result;
}
