/*
 *  Copyright (C) 2005, DivX Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
*/
/**
    \file aacframe.h
*/

#ifndef _AAC_FRAME_H_
#define _AAC_FRAME_H_

#include "common.h"

typedef enum {
    AAC_1_AND_1 = 0,
    AAC_CENTER,
    AAC_LEFT_RIGHT,
    AAC_LEFT_CENTER_RIGHT,
    AAC_LEFT_RIGHT_SURROUND,
    AAC_LEFT_CENTER_RIGHT_SURROUND,
    AAC_LEFT_SURROUND_RIGHT_SURROUND,
    AAC_CENTER_LEFT_SURROUND_RIGHT_SURROUND,
    AAC_Channel_Size
} Aac_Channel;

class AAC_Frame {
public:
	AAC_Frame(); /// \todo initialise class variables
	bool Eval(const uint8_t *header, size_t HeadSize);
	bool IsValid() const {return bValid;}
	uint16_t SampleSize() const {return 1536;}
	uint16_t SamplingRate() const {return mySamplingRate;}
	uint16_t DataSize() const;
	uint16_t Channels() const;// {return (myChannels == MPEG_CHANNEL_MONO)?1:2;}
	bool HasCrc() const { return myCopyrightOn;}
	Aac_Channel ChannelMode() const {return myChannels;}
	uint16_t BitRate() const {return myBitRate;}
	
	inline operator const uint8_t *() const
	{
	    return myBinaryValue;
	}

	void setChannelMode(const Aac_Channel value) {myChannels = value;}

	bool Compatible(const AAC_Frame & aFrame) const;

protected:
	Aac_Channel   myChannels;
	bool          myLfeChannel;
	int16_t         myBitRate;
	uint16_t        mySamplingRate;
	uint16_t        myCrc1;
	uint16_t        myCrc2;
    uint16_t        myFrameSize;
	bool          bValid;
	bool          myPaddingOn;
	bool          myPrivateOn;
	bool          myCopyrightOn;
	bool          myOriginalOn;

	uint8_t        myBinaryValue[8];

	static const uint32_t SamplingRates[12];
	static const int16_t  Bitrates[20];
};

#endif // _AAC_FRAME_H_
