/*
 * Generic AC3 frame parser
 * Copyright (c) 2005 DivX, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
/**
 * @file ac3frame.h
 */

#ifndef _AC3_FRAME_H_
#define _AC3_FRAME_H_

#include "common.h"

typedef enum {
    AC3_1_AND_1 = 0,
    AC3_CENTER,
    AC3_LEFT_RIGHT,
    AC3_LEFT_CENTER_RIGHT,
    AC3_LEFT_RIGHT_SURROUND,
    AC3_LEFT_CENTER_RIGHT_SURROUND,
    AC3_LEFT_SURROUND_RIGHT_SURROUND,
    AC3_CENTER_LEFT_SURROUND_RIGHT_SURROUND,
    AC3_Channel_Size
} Ac3_Channel;

class AC3_Frame {
    public:
	AC3_Frame(); /// \todo initialise class variables
	bool Eval(const uint8_t header[5]);
	bool IsValid() const {return bValid;}
	uint16_t SampleSize() const {return 1536;}
	uint16_t SamplingRate() const {return mySamplingRate;}
	uint16_t DataSize() const;
	uint16_t Channels() const;// {return (myChannels == MPEG_CHANNEL_MONO)?1:2;}
	bool HasCrc() const { return myCopyrightOn;}
	Ac3_Channel ChannelMode() const {return myChannels;}
	uint16_t BitRate() const {return myBitRate;}
	
	inline operator const uint8_t *() const
	{
	    return myBinaryValue;
	}

	void setChannelMode(const Ac3_Channel value) {myChannels = value;}

	bool Compatible(const AC3_Frame & aFrame) const;
    protected:
	Ac3_Channel   myChannels;
	bool          myLfeChannel;
	int16_t         myBitRate;
	uint16_t        mySamplingRate;
	uint16_t        myCrc1;
	uint16_t        myCrc2;
    uint16_t        myFrameSize;
	bool          bValid;
	bool          myPaddingOn;
	bool          myPrivateOn;
	bool          myCopyrightOn;
	bool          myOriginalOn;

	uint8_t        myBinaryValue[8];

	static const uint32_t SamplingRates[4];
	static const int16_t  Bitrates[20];
};

#endif // _AC3_FRAME_H_
