/*
 * DirectShow capture io for ffmpeg system
 * Copyright (c) 2005 DivX Inc
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */

#ifndef _DSHOW_CAP_GRAPH_H_
#define _DSHOW_CAP_GRAPH_H_

#include <vector>
#include <string>

#include <windows.h>
#include <dshow.h>

#include "avformat.h"

typedef std::vector<std::wstring> StringList;

class SinkCallBack;

class DShowCaptureGraph {
public:
	DShowCaptureGraph(AVFormatContext *avctx, const char *device);
	virtual ~DShowCaptureGraph();

	virtual bool FillCodec(AVFormatParameters & avfctx) = 0;

	static const char * GetPhysicalInput(long physical);
	static PhysicalConnectorType GetPhysicalType(const char *pin_type);

	static IBaseFilter *FindDevice( const wchar_t *device, const GUID & DeviceType, AVFormatContext *avctx = NULL );
	static size_t GetDeviceList( StringList & out_list, const GUID & DeviceType, AVFormatContext *avctx = NULL );

	HRESULT AttachFiltergraph();

protected:
	enum PLAYSTATE {Stopped, Paused, Running, Init};

	HRESULT GetInterfaces(void);
	void CloseInterfaces(void);

	struct PhysicalCon {
		PhysicalConnectorType physcon;
		const char *physname;
	};
	static const PhysicalCon PhysConList[];

	IMediaControl *g_pMC;
	IGraphBuilder *g_pGraph;
	ICaptureGraphBuilder2 *g_pCapture;
	PLAYSTATE g_psCurrent;
	AVFormatContext *m_avctx;
	char *m_devicename;
	WCHAR *m_OutputPinName;
	AVStream *m_Stream;
	DWORD dwRotId;

	SinkCallBack *m_Sink;
};

#endif // _DSHOW_CAP_GRAPH_H_
