/*
 *  Copyright (C) 2005, DivX Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
*/
/**
    \file aacframe.h
*/

#ifndef _DTS_FRAME_H_
#define _DTS_FRAME_H_

#include "common.h"

typedef enum {
    DTS_1 = 0,
    DTS_1_AND_1,
    DTS_LEFT_RIGHT,
    DTS_LEFT_RIGHT_SUM_DIFF,
    DTS_LEFT_RIGHT_TOTAL,
    DTS_CENTER_LEFT_RIGHT,
    DTS_LEFT_RIGHT_SURROUND,
    DTS_CENTER_LEFT_RIGHT_SURROUND,
    DTS_LEFT_RIGHT_SURROUND_LEFT_SURROUND_RIGHT,
    DTS_CENTER_LEFT_RIGHT_SURROUND_LEFT_SURROUND_RIGHT,
    DTS_CENTER_LEFT_CENTER_RIGHT_LEFT_RIGHT_SURROUND_LEFT_SURROUND_RIGHT,
    DTS_CENTER_LEFT_RIGHT_LEFT_REAR_RIGHT_REAR_OVERHEAD,
    DTS_CENTER_FRONT_CENTER_RIGHT_FRONT_LEFT_FRONT_RIGHT_LEFT_REAR_RIGHT_REAR,
    DTS_CENTER_LEFT_CENTER_CENTER_RIGHT_LEFT_RIGHT_SURROUND_LEFT_SURROUND_RIGHT,
    DTS_CENTER_LEFT_CENTER_RIGHT_LEFT_RIGHT_SURROUND_LEFT1_SURROUND_LEFT2_SURROUND_RIGHT1_SURROUND_RIGHT2,
    DTS_CENTER_LEFT_CENTER_CENTER_RIGHT_LEFT_RIGHT_SURROUND_LEFT_SURROUND_SURROUND_RIGHT,
    DTS_Channel_Size
} Dts_Channel;

class DTS_Frame {
public:
	DTS_Frame(); /// \todo initialise class variables
	bool Eval(const uint8_t *header, size_t HeadSize);
	bool IsValid() const {return bValid;}
	uint16_t SampleSize() const {return 1536;}
	uint16_t SamplingRate() const;
	uint16_t DataSize() const;
	uint16_t Channels() const;// {return (myChannels == MPEG_CHANNEL_MONO)?1:2;}
	bool HasCrc() const { return myCopyrightOn;}
	Dts_Channel ChannelMode() const {return myChannels;}
	uint16_t BitRate() const {return myBitRate;}
	
	inline operator const uint8_t *() const
	{
	    return myBinaryValue;
	}

	void setChannelMode(const Dts_Channel value) {myChannels = value;}

	bool Compatible(const DTS_Frame & aFrame) const;

protected:
	Dts_Channel   myChannels;
	int16_t         myBitRate;
	uint16_t        mySamplingRate;
    uint16_t        myFrameSize;
	bool          bValid;
	bool          myPaddingOn;
	bool          myPrivateOn;
	bool          myCopyrightOn;
	bool          myOriginalOn;

	uint8_t        myBinaryValue[10];

	static const int32_t SamplingRates[16];
	static const int16_t  Bitrates[32];
};

#endif // _DTS_FRAME_H_
